#ifndef LIGHINGENV_H
#define LIGHINGENV_H

#include <glm/vec3.hpp>

class Texture;

class LighingEnv
{
    Texture* m_radianceMap;
    Texture* m_irradianceMap;

    bool m_hasFog;
    float m_fogDensity;
    glm::vec3 m_fogColor;

    bool m_isGroundFog;
    float m_fogFloor;
    float m_fogCeiling;

public:
    LighingEnv();

    // environment mapping methods

    void setRadianceMap(Texture* tex); // used for the specular component
    void setIrradianceMap(Texture* tex); // used for the diffuse component

    // fog methods

    void setNoFog() { m_hasFog = false; }

    void setUniformFog(float fogDensity, glm::vec3 fogColor)
    { m_hasFog = true; m_fogDensity = fogDensity; m_fogColor = fogColor; }

    void setGroundFog(float fogDensity, glm::vec3 fogColor, float fogFloor, float fogCeiling)
    { setUniformFog(fogDensity, fogColor); m_isGroundFog = true; m_fogFloor = fogFloor; m_fogCeiling = fogCeiling; }

    /**
     * @brief this gui does not include the begin/end calls, only the fog editor
     */
    void gui();
};

#endif // LIGHINGENV_H
